package com.bomostory.sceneeditmodule.screen.movie

import android.app.AlertDialog
import android.app.ProgressDialog
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.DialogInterface
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.os.Handler
import android.support.v4.app.DialogFragment
import android.support.v7.app.AppCompatActivity
import android.view.View
import android.widget.CompoundButton
import android.widget.SeekBar
import android.widget.Toast
import com.bomostory.sceneeditmodule.Config
import com.bomostory.sceneeditmodule.PdfMaker
import com.bomostory.sceneeditmodule.SuperMovieMaker
import com.bomostory.sceneeditmodule.basicdata.*
import com.bomostory.sceneeditmodule.screen.movie.music.MusicEditDialog
import com.bomostory.sceneeditmodule.screen.movie.music.MusicSelectDialog
import com.bomostory.sceneeditmodule.screen.movie.music.MusicSelectFragment
import com.bomostory.sceneeditmodule.share.ShareDialog
import com.bomostory.sceneeditmodule.screen.view.AudioTrackGroupView
import com.bomostory.sceneeditmodule.screen.view.MovieSelectView
import com.bomostory.sceneeditmodule.share.ExportPdfDialog
import com.bomostory.sceneeditmodule.utils.MoviePlayerV2
import com.bomostory.sceneeditmodule.utils.MusicPlayerV2
import com.example.exportmedia.MediaHelper
import com.example.tfat.myapplication.R
import com.google.gson.Gson
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.activity_movie_edit.*
import java.io.File


class MovieEditActivity : AppCompatActivity(),
        DialogInterface.OnDismissListener,
        MusicSelectFragment.OnFragmentInteractionListener,
        MusicEditDialog.OnFragmentInteractionListener {

    override fun onMusicImportClick(music: Music) {
        moviePlayer.init()

        val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
        musics.add(music)
        viewModel.musicListLiveData.value = musics
        viewModel.musicLiveData.value = music

        val dialogFragment = supportFragmentManager.findFragmentByTag(MUSIC_SELECT_DIALOG_TAG) as DialogFragment
        dialogFragment.dismiss()
    }

    override fun onMusicPreCastClick(music: Music) {
        musicPlayer.apply {
            this.music = music
            stop()
            init()
            start()
        }
    }

    override fun onMusicPlayClick(music: Music, startPosition: Int) {
        musicPlayer.apply {
            this.music = music
            playChangedListener = object : MusicPlayerV2.OnPlayChangedListener {
                override fun OnPlayChanged(position: Int) {
                    runOnUiThread {
                        musicEditDialog?.currentPosition = position
                    }
                }
            }
            init()
            seekTo(startPosition)
            start()
        }
    }

    override fun onMusicVolumeChanged(volume: Float) {
        musicPlayer.setVolume(volume)
    }

    override fun onMusicPauseClick() {
        musicPlayer.pause()
    }

    override fun onMusicStop() {
        musicPlayer.stop()
    }

    override fun onMusicEditFinish(music: Music) {
        moviePlayer.init()

        //TODO change path to correct path
        val outputFile = File(Environment.getExternalStorageDirectory(), "test${System.currentTimeMillis()}.mp3")

        val pd = ProgressDialog(this).apply {
            setMessage("progressing")
            setCancelable(false)
        }

        viewModel.exportEditAudio(music, outputFile)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe {
                    pd.show()
                }
                .doFinally {
                    pd.dismiss()
                }
                .subscribe({
                    music.path = outputFile.path

                    val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
                    musics[musics.indexOf(viewModel.musicLiveData.value)] = music
                    viewModel.musicListLiveData.value = musics
                    viewModel.musicLiveData.value = music
                }, {

                })
    }

    override fun onDismiss(dialog: DialogInterface?) {
        musicPlayer.stop()
        delayedHide(100)
    }

    private var musicEditDialog: MusicEditDialog? = null
    private val shareDialog = ShareDialog()

    private val mHideHandler = Handler()
    private val mHidePart2Runnable = Runnable {
        window.decorView.systemUiVisibility =
                View.SYSTEM_UI_FLAG_LOW_PROFILE or
                View.SYSTEM_UI_FLAG_FULLSCREEN or
                View.SYSTEM_UI_FLAG_LAYOUT_STABLE or
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY or
                View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or
                View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
    }
    private val mShowPart2Runnable = Runnable {
        supportActionBar?.show()
    }
    private val mHideRunnable = Runnable {
        hide()
    }

    private val moviePlayer = MoviePlayerV2()

    private val musicPlayer = MusicPlayerV2()

    private val mediaHelper = MediaHelper()

    private val editMoviePlayListener = object : MoviePlayerV2.OnMoviePlayListener {
        override fun onMovieSceneUpdate(scene: Scene) {
            runOnUiThread {
                if (viewModel.sceneLiveData.value != scene) {
                    viewModel.sceneLiveData.value = scene
                }
            }
        }

        override fun onMoviePlayComplete() {
            runOnUiThread {
                movieEditView.isBtnPanelEnable = true
            }
        }

        override fun onMoviePlayViewUpdate(x: Int) {
            runOnUiThread {
                movieView.x = x
            }
        }

        override fun onMoviePlayTimeUpdate(time: Long) {
            runOnUiThread {
                movieEditView.playTime = time
            }
        }
    }

    private val dialogMoviePlayListener = object : MoviePlayerV2.OnMoviePlayListener {
        override fun onMovieSceneUpdate(scene: Scene) {
            runOnUiThread {
                if (shareDialog.scene != scene)
                    shareDialog.scene = scene
            }
        }

        override fun onMoviePlayViewUpdate(x: Int) {
            runOnUiThread {
                shareDialog.x = x
            }
        }

        override fun onMoviePlayTimeUpdate(time: Long) {
            runOnUiThread {
                shareDialog.time = time
            }
        }

        override fun onMoviePlayComplete() {
        }
    }

    companion object {
        const val PROJECT_KEY = "project_key"
        const val MUSIC_SELECT_DIALOG_TAG = "music_select_dialog_tag"
        const val MUSIC_EDIT_DIALOG_TAG = "music_edit_dialog_tag"
        const val FPS: Int = 30
        private const val UI_ANIMATION_DELAY = 300
    }

    private lateinit var viewModel: MovieEditViewModel

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_movie_edit)

        mediaHelper.init(this)

        viewModel = ViewModelProviders.of(this).get(MovieEditViewModel::class.java)
        viewModel.mediaHelper = mediaHelper
//        viewModel.project = Gson().fromJson<Project>(intent.extras.getString(PROJECT_KEY), Project::class.java)
        viewModel.project = Gson().fromJson<Project>("{\"actorResource\":[[\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act4/actor_lfsdbuf04_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act11/actor_lfsdbuf11_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act9/actor_lfsdbuf09_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act8/actor_lfsdbuf08_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act6/actor_lfsdbuf06_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act5/actor_lfsdbuf05_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act10/actor_lfsdbuf10_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act7/actor_lfsdbuf07_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act2/actor_lfsdbuf02_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act12/actor_lfsdbuf12_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act3/actor_lfsdbuf03_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act1/actor_lfsdbuf01_small.png\"]],\"assetFolder\":{\"path\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset\"},\"coverFile\":{\"path\":\"/storage/emulated/0/Bomo/Projects/ffgg/cover.png\"},\"name\":\"ffgg\",\"projectFile\":{\"path\":\"/storage/emulated/0/Bomo/Projects/ffgg/index\"},\"projectFolder\":{\"path\":\"/storage/emulated/0/Bomo/Projects/ffgg\"},\"scene1File\":{\"path\":\"/storage/emulated/0/Bomo/Projects/ffgg/scene1.png\"},\"scene2File\":{\"path\":\"/storage/emulated/0/Bomo/Projects/ffgg/scene2.png\"},\"sceneResource\":[\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/scene/s1/scene_lfsample01_small.png\",\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/scene/s32/scene_lfsky32_small.png\"],\"story\":{\"scenes\":[{\"backgroundName\":\"s1\",\"backgroundPath\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/scene/s1/scene_lfsample01_large.png\",\"layers\":[{\"actors\":[{\"isMoveable\":false,\"isSelect\":false,\"parentLayerIndex\":0,\"positionX\":746,\"positionY\":210,\"positionZ\":0,\"resourcePath\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act4/actor_lfsdbuf04_small.png\",\"sideHeight\":558,\"sideLength\":457},{\"isMoveable\":false,\"isSelect\":false,\"parentLayerIndex\":0,\"positionX\":529,\"positionY\":451,\"positionZ\":1,\"resourcePath\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act5/actor_lfsdbuf05_small.png\",\"sideHeight\":200,\"sideLength\":200},{\"isMoveable\":false,\"isSelect\":false,\"parentLayerIndex\":0,\"positionX\":1228,\"positionY\":373,\"positionZ\":2,\"resourcePath\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act2/actor_lfsdbuf02_small.png\",\"sideHeight\":200,\"sideLength\":200},{\"isMoveable\":false,\"isSelect\":false,\"parentLayerIndex\":0,\"positionX\":1235,\"positionY\":589,\"positionZ\":3,\"resourcePath\":\"/storage/emulated/0/Bomo/Assets/T15-FREE/asset/T15-FREE/actor/Life_04dysf_color/act7/actor_lfsdbuf07_small.png\",\"sideHeight\":200,\"sideLength\":200}],\"id\":0},{\"actors\":[],\"id\":1},{\"actors\":[],\"id\":2},{\"actors\":[],\"id\":3},{\"actors\":[],\"id\":4}],\"record\":{\"period\":16797,\"tracks\":[{\"positionX\":0,\"time\":740},{\"positionX\":-25,\"time\":757},{\"positionX\":-45,\"time\":775},{\"positionX\":-68,\"time\":792},{\"positionX\":-92,\"time\":808},{\"positionX\":-118,\"time\":824},{\"positionX\":-144,\"time\":841},{\"positionX\":-175,\"time\":858},{\"positionX\":-206,\"time\":875},{\"positionX\":-236,\"time\":892},{\"positionX\":-264,\"time\":908},{\"positionX\":-297,\"time\":924},{\"positionX\":-324,\"time\":941},{\"positionX\":-354,\"time\":958},{\"positionX\":-381,\"time\":975},{\"positionX\":-408,\"time\":992},{\"positionX\":-433,\"time\":1008},{\"positionX\":-453,\"time\":1024},{\"positionX\":-472,\"time\":1042},{\"positionX\":-487,\"time\":1059},{\"positionX\":-498,\"time\":1075},{\"positionX\":-505,\"time\":1092},{\"positionX\":-509,\"time\":1109},{\"positionX\":-510,\"time\":1124},{\"positionX\":-511,\"time\":1141},{\"positionX\":-510,\"time\":1159},{\"positionX\":-499,\"time\":1175},{\"positionX\":-478,\"time\":1192},{\"positionX\":-455,\"time\":1209},{\"positionX\":-429,\"time\":1226},{\"positionX\":-403,\"time\":1242},{\"positionX\":-373,\"time\":1259},{\"positionX\":-345,\"time\":1276},{\"positionX\":-312,\"time\":1293},{\"positionX\":-278,\"time\":1309},{\"positionX\":-244,\"time\":1327},{\"positionX\":-202,\"time\":1343},{\"positionX\":-168,\"time\":1360},{\"positionX\":-131,\"time\":1375},{\"positionX\":-93,\"time\":1393},{\"positionX\":-55,\"time\":1409},{\"positionX\":-20,\"time\":1426},{\"positionX\":15,\"time\":1443},{\"positionX\":49,\"time\":1459},{\"positionX\":79,\"time\":1475},{\"positionX\":108,\"time\":1492},{\"positionX\":133,\"time\":1510},{\"positionX\":160,\"time\":1527},{\"positionX\":181,\"time\":1543},{\"positionX\":201,\"time\":1560},{\"positionX\":223,\"time\":1576},{\"positionX\":241,\"time\":1593},{\"positionX\":262,\"time\":1609},{\"positionX\":280,\"time\":1626},{\"positionX\":295,\"time\":1643},{\"positionX\":311,\"time\":1659},{\"positionX\":327,\"time\":1676},{\"positionX\":340,\"time\":1693},{\"positionX\":348,\"time\":1710},{\"positionX\":356,\"time\":1727},{\"positionX\":363,\"time\":1743},{\"positionX\":369,\"time\":1760},{\"positionX\":373,\"time\":1776},{\"positionX\":375,\"time\":1794},{\"positionX\":376,\"time\":1810},{\"positionX\":376,\"time\":1826},{\"positionX\":376,\"time\":1842},{\"positionX\":376,\"time\":1859},{\"positionX\":374,\"time\":1876},{\"positionX\":372,\"time\":1894},{\"positionX\":369,\"time\":1911},{\"positionX\":362,\"time\":1927},{\"positionX\":353,\"time\":1943},{\"positionX\":339,\"time\":1961},{\"positionX\":324,\"time\":1977},{\"positionX\":308,\"time\":1994},{\"positionX\":290,\"time\":2010},{\"positionX\":267,\"time\":2026},{\"positionX\":245,\"time\":2043},{\"positionX\":219,\"time\":2060},{\"positionX\":200,\"time\":2077},{\"positionX\":175,\"time\":2093},{\"positionX\":153,\"time\":2110},{\"positionX\":131,\"time\":2127},{\"positionX\":107,\"time\":2144},{\"positionX\":89,\"time\":2161},{\"positionX\":68,\"time\":2178},{\"positionX\":49,\"time\":2194},{\"positionX\":33,\"time\":2211},{\"positionX\":16,\"time\":2228},{\"positionX\":-2,\"time\":2244},{\"positionX\":-18,\"time\":2261},{\"positionX\":-35,\"time\":2277},{\"positionX\":-54,\"time\":2294},{\"positionX\":-70,\"time\":2311},{\"positionX\":-85,\"time\":2327},{\"positionX\":-102,\"time\":2344},{\"positionX\":-119,\"time\":2361},{\"positionX\":-136,\"time\":2378},{\"positionX\":-153,\"time\":2395},{\"positionX\":-169,\"time\":2412},{\"positionX\":-184,\"time\":2428},{\"positionX\":-201,\"time\":2445},{\"positionX\":-217,\"time\":2461},{\"positionX\":-229,\"time\":2478},{\"positionX\":-242,\"time\":2494},{\"positionX\":-254,\"time\":2512},{\"positionX\":-264,\"time\":2529},{\"positionX\":-273,\"time\":2545},{\"positionX\":-279,\"time\":2562},{\"positionX\":-284,\"time\":2579},{\"positionX\":-288,\"time\":2595},{\"positionX\":-290,\"time\":2612},{\"positionX\":-292,\"time\":2629},{\"positionX\":-294,\"time\":2646},{\"positionX\":-294,\"time\":2661},{\"positionX\":-294,\"time\":2678},{\"positionX\":-290,\"time\":2695},{\"positionX\":-282,\"time\":2711},{\"positionX\":-271,\"time\":2729},{\"positionX\":-256,\"time\":2746},{\"positionX\":-240,\"time\":2762},{\"positionX\":-221,\"time\":2778},{\"positionX\":-201,\"time\":2796},{\"positionX\":-183,\"time\":2813},{\"positionX\":-159,\"time\":2829},{\"positionX\":-137,\"time\":2845},{\"positionX\":-113,\"time\":2863},{\"positionX\":-91,\"time\":2880},{\"positionX\":-70,\"time\":2896},{\"positionX\":-47,\"time\":2913},{\"positionX\":-30,\"time\":2930},{\"positionX\":-8,\"time\":2946},{\"positionX\":12,\"time\":2963},{\"positionX\":29,\"time\":2980},{\"positionX\":49,\"time\":2996},{\"positionX\":67,\"time\":3013},{\"positionX\":81,\"time\":3029},{\"positionX\":100,\"time\":3046},{\"positionX\":116,\"time\":3063},{\"positionX\":130,\"time\":3080},{\"positionX\":143,\"time\":3096},{\"positionX\":159,\"time\":3113},{\"positionX\":173,\"time\":3129},{\"positionX\":184,\"time\":3146},{\"positionX\":195,\"time\":3163},{\"positionX\":206,\"time\":3180},{\"positionX\":216,\"time\":3197},{\"positionX\":226,\"time\":3213},{\"positionX\":234,\"time\":3230},{\"positionX\":242,\"time\":3246},{\"positionX\":251,\"time\":3263},{\"positionX\":260,\"time\":3280},{\"positionX\":270,\"time\":3296},{\"positionX\":282,\"time\":3314},{\"positionX\":289,\"time\":3331},{\"positionX\":299,\"time\":3347},{\"positionX\":310,\"time\":3364},{\"positionX\":321,\"time\":3381},{\"positionX\":332,\"time\":3397},{\"positionX\":342,\"time\":3414},{\"positionX\":350,\"time\":3431},{\"positionX\":359,\"time\":3447},{\"positionX\":366,\"time\":3464},{\"positionX\":368,\"time\":3474},{\"positionX\":368,\"time\":4410},{\"positionX\":346,\"time\":4433},{\"positionX\":332,\"time\":4449},{\"positionX\":317,\"time\":4467},{\"positionX\":302,\"time\":4483},{\"positionX\":280,\"time\":4500},{\"positionX\":260,\"time\":4518},{\"positionX\":237,\"time\":4533},{\"positionX\":213,\"time\":4550},{\"positionX\":189,\"time\":4567},{\"positionX\":162,\"time\":4584},{\"positionX\":137,\"time\":4600},{\"positionX\":108,\"time\":4617},{\"positionX\":79,\"time\":4634},{\"positionX\":50,\"time\":4651},{\"positionX\":22,\"time\":4667},{\"positionX\":-6,\"time\":4684},{\"positionX\":-32,\"time\":4701},{\"positionX\":-57,\"time\":4717},{\"positionX\":-78,\"time\":4734},{\"positionX\":-101,\"time\":4750},{\"positionX\":-119,\"time\":4767},{\"positionX\":-140,\"time\":4785},{\"positionX\":-160,\"time\":4801},{\"positionX\":-180,\"time\":4817},{\"positionX\":-202,\"time\":4834},{\"positionX\":-225,\"time\":4851},{\"positionX\":-246,\"time\":4868},{\"positionX\":-268,\"time\":4884},{\"positionX\":-286,\"time\":4900},{\"positionX\":-304,\"time\":4918},{\"positionX\":-319,\"time\":4935},{\"positionX\":-331,\"time\":4951},{\"positionX\":-339,\"time\":4968},{\"positionX\":-343,\"time\":4984},{\"positionX\":-346,\"time\":5001},{\"positionX\":-348,\"time\":5018},{\"positionX\":-349,\"time\":5034},{\"positionX\":-347,\"time\":5051},{\"positionX\":-342,\"time\":5068},{\"positionX\":-327,\"time\":5085},{\"positionX\":-308,\"time\":5102},{\"positionX\":-287,\"time\":5118},{\"positionX\":-262,\"time\":5135},{\"positionX\":-234,\"time\":5152},{\"positionX\":-202,\"time\":5168},{\"positionX\":-170,\"time\":5185},{\"positionX\":-137,\"time\":5202},{\"positionX\":-97,\"time\":5219},{\"positionX\":-60,\"time\":5235},{\"positionX\":-21,\"time\":5251},{\"positionX\":22,\"time\":5268},{\"positionX\":58,\"time\":5285},{\"positionX\":94,\"time\":5302},{\"positionX\":130,\"time\":5318},{\"positionX\":160,\"time\":5336},{\"positionX\":187,\"time\":5352},{\"positionX\":210,\"time\":5369},{\"positionX\":231,\"time\":5386},{\"positionX\":246,\"time\":5402},{\"positionX\":260,\"time\":5418},{\"positionX\":275,\"time\":5435},{\"positionX\":286,\"time\":5453},{\"positionX\":288,\"time\":5463},{\"positionX\":260,\"time\":9884},{\"positionX\":245,\"time\":9893},{\"positionX\":216,\"time\":9914},{\"positionX\":190,\"time\":9932},{\"positionX\":160,\"time\":9947},{\"positionX\":127,\"time\":9965},{\"positionX\":93,\"time\":9981},{\"positionX\":52,\"time\":9998},{\"positionX\":6,\"time\":10014},{\"positionX\":-39,\"time\":10031},{\"positionX\":-86,\"time\":10048},{\"positionX\":-132,\"time\":10064},{\"positionX\":-183,\"time\":10081},{\"positionX\":-230,\"time\":10098},{\"positionX\":-279,\"time\":10115},{\"positionX\":-329,\"time\":10131},{\"positionX\":-378,\"time\":10147},{\"positionX\":-425,\"time\":10164},{\"positionX\":-470,\"time\":10181},{\"positionX\":-513,\"time\":10198},{\"positionX\":-551,\"time\":10215},{\"positionX\":-588,\"time\":10231},{\"positionX\":-620,\"time\":10249},{\"positionX\":-644,\"time\":10265},{\"positionX\":-665,\"time\":10281},{\"positionX\":-681,\"time\":10298},{\"positionX\":-691,\"time\":10314},{\"positionX\":-695,\"time\":10332},{\"positionX\":-697,\"time\":10348},{\"positionX\":-698,\"time\":10364},{\"positionX\":-692,\"time\":10381},{\"positionX\":-675,\"time\":10398},{\"positionX\":-650,\"time\":10415},{\"positionX\":-617,\"time\":10431},{\"positionX\":-580,\"time\":10448},{\"positionX\":-544,\"time\":10465},{\"positionX\":-504,\"time\":10482},{\"positionX\":-455,\"time\":10498},{\"positionX\":-408,\"time\":10515},{\"positionX\":-365,\"time\":10532},{\"positionX\":-319,\"time\":10548},{\"positionX\":-273,\"time\":10565},{\"positionX\":-226,\"time\":10582},{\"positionX\":-178,\"time\":10598},{\"positionX\":-131,\"time\":10616},{\"positionX\":-84,\"time\":10632},{\"positionX\":-42,\"time\":10649},{\"positionX\":-3,\"time\":10665},{\"positionX\":40,\"time\":10683},{\"positionX\":83,\"time\":10700},{\"positionX\":120,\"time\":10716},{\"positionX\":157,\"time\":10733},{\"positionX\":192,\"time\":10750},{\"positionX\":224,\"time\":10766},{\"positionX\":255,\"time\":10783},{\"positionX\":284,\"time\":10800},{\"positionX\":309,\"time\":10816},{\"positionX\":334,\"time\":10833},{\"positionX\":356,\"time\":10850},{\"positionX\":372,\"time\":10866},{\"positionX\":388,\"time\":10883},{\"positionX\":401,\"time\":10900},{\"positionX\":410,\"time\":10917},{\"positionX\":414,\"time\":10933},{\"positionX\":418,\"time\":10950},{\"positionX\":419,\"time\":10967},{\"positionX\":419,\"time\":10983},{\"positionX\":419,\"time\":10999},{\"positionX\":418,\"time\":11016},{\"positionX\":409,\"time\":11034},{\"positionX\":394,\"time\":11050},{\"positionX\":376,\"time\":11067},{\"positionX\":349,\"time\":11084},{\"positionX\":323,\"time\":11101},{\"positionX\":290,\"time\":11117},{\"positionX\":261,\"time\":11134},{\"positionX\":222,\"time\":11150},{\"positionX\":182,\"time\":11167},{\"positionX\":148,\"time\":11183},{\"positionX\":108,\"time\":11201},{\"positionX\":66,\"time\":11217},{\"positionX\":25,\"time\":11234},{\"positionX\":-16,\"time\":11251},{\"positionX\":-55,\"time\":11267},{\"positionX\":-93,\"time\":11284},{\"positionX\":-132,\"time\":11301},{\"positionX\":-164,\"time\":11318},{\"positionX\":-198,\"time\":11334},{\"positionX\":-226,\"time\":11351},{\"positionX\":-252,\"time\":11368},{\"positionX\":-272,\"time\":11384},{\"positionX\":-290,\"time\":11401},{\"positionX\":-305,\"time\":11418},{\"positionX\":-314,\"time\":11435},{\"positionX\":-320,\"time\":11451},{\"positionX\":-323,\"time\":11468},{\"positionX\":-324,\"time\":11485},{\"positionX\":-325,\"time\":11501},{\"positionX\":-325,\"time\":11517},{\"positionX\":-324,\"time\":11534},{\"positionX\":-301,\"time\":14006},{\"positionX\":-272,\"time\":14024},{\"positionX\":-245,\"time\":14042},{\"positionX\":-208,\"time\":14059},{\"positionX\":-169,\"time\":14076},{\"positionX\":-130,\"time\":14092},{\"positionX\":-91,\"time\":14108},{\"positionX\":-47,\"time\":14124},{\"positionX\":-3,\"time\":14141},{\"positionX\":41,\"time\":14158},{\"positionX\":87,\"time\":14175},{\"positionX\":133,\"time\":14192},{\"positionX\":180,\"time\":14208},{\"positionX\":226,\"time\":14225},{\"positionX\":272,\"time\":14242},{\"positionX\":314,\"time\":14259},{\"positionX\":354,\"time\":14275},{\"positionX\":395,\"time\":14292},{\"positionX\":428,\"time\":14309},{\"positionX\":457,\"time\":14325},{\"positionX\":485,\"time\":14342},{\"positionX\":506,\"time\":14359},{\"positionX\":524,\"time\":14376},{\"positionX\":539,\"time\":14392},{\"positionX\":548,\"time\":14408},{\"positionX\":553,\"time\":14425},{\"positionX\":557,\"time\":14442},{\"positionX\":558,\"time\":14459},{\"positionX\":558,\"time\":14476},{\"positionX\":557,\"time\":14492},{\"positionX\":548,\"time\":14509},{\"positionX\":528,\"time\":14525},{\"positionX\":508,\"time\":14542},{\"positionX\":477,\"time\":14559},{\"positionX\":448,\"time\":14576},{\"positionX\":414,\"time\":14593},{\"positionX\":381,\"time\":14609},{\"positionX\":346,\"time\":14626},{\"positionX\":308,\"time\":14643},{\"positionX\":271,\"time\":14660},{\"positionX\":237,\"time\":14677},{\"positionX\":200,\"time\":14693},{\"positionX\":162,\"time\":14710},{\"positionX\":130,\"time\":14727},{\"positionX\":95,\"time\":14743},{\"positionX\":66,\"time\":14759},{\"positionX\":39,\"time\":14777},{\"positionX\":15,\"time\":14794},{\"positionX\":-7,\"time\":14810},{\"positionX\":-23,\"time\":14827},{\"positionX\":-37,\"time\":14844},{\"positionX\":-48,\"time\":14860},{\"positionX\":-56,\"time\":14877},{\"positionX\":-61,\"time\":14894},{\"positionX\":-63,\"time\":14910},{\"positionX\":-65,\"time\":14928},{\"positionX\":-65,\"time\":14944},{\"positionX\":-65,\"time\":14960},{\"positionX\":-65,\"time\":14976},{\"positionX\":-61,\"time\":14993},{\"positionX\":-58,\"time\":15011},{\"positionX\":-53,\"time\":15028},{\"positionX\":-50,\"time\":15039}]},\"sceneWidth\":2048}]},\"themeAssetIndex\":{\"contains\":{\"actor\":{\"Life_04dysf_color\":{\"act4\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act4/actor_lfsdbuf04_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act4/actor_lfsdbuf04_small.png\"},\"act11\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act11/actor_lfsdbuf11_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act11/actor_lfsdbuf11_small.png\"},\"act9\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act9/actor_lfsdbuf09_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act9/actor_lfsdbuf09_small.png\"},\"act8\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act8/actor_lfsdbuf08_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act8/actor_lfsdbuf08_small.png\"},\"act6\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act6/actor_lfsdbuf06_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act6/actor_lfsdbuf06_small.png\"},\"act5\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act5/actor_lfsdbuf05_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act5/actor_lfsdbuf05_small.png\"},\"act2\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act2/actor_lfsdbuf02_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act2/actor_lfsdbuf02_small.png\"},\"act7\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act7/actor_lfsdbuf07_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act7/actor_lfsdbuf07_small.png\"},\"act10\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act10/actor_lfsdbuf10_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act10/actor_lfsdbuf10_small.png\"},\"act12\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act12/actor_lfsdbuf12_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act12/actor_lfsdbuf12_small.png\"},\"act3\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act3/actor_lfsdbuf03_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act3/actor_lfsdbuf03_small.png\"},\"act1\":{\"large_file\":\"T15-FREE/actor/Life_04dysf_color/act1/actor_lfsdbuf01_large.png\",\"small_file\":\"T15-FREE/actor/Life_04dysf_color/act1/actor_lfsdbuf01_small.png\"}}},\"music\":{\"muisland02\":\"T15-FREE/music/muisland02.mp3\",\"muisland01\":\"T15-FREE/music/muisland01.mp3\"},\"object\":{\"o56\":{\"large_file\":\"T15-FREE/object/o56/object_lftree56_large.png\",\"small_file\":\"T15-FREE/object/o56/object_lftree56_small.png\"},\"o68\":{\"large_file\":\"T15-FREE/object/o68/scene_lfw68_large.png\",\"small_file\":\"T15-FREE/object/o68/scene_lfw68_small.png\"},\"o95\":{\"large_file\":\"T15-FREE/object/o95/scene_lfw95_large.png\",\"small_file\":\"T15-FREE/object/o95/scene_lfw95_small.png\"},\"o53\":{\"large_file\":\"T15-FREE/object/o53/object_lftree53_large.png\",\"small_file\":\"T15-FREE/object/o53/object_lftree53_small.png\"},\"o9\":{\"large_file\":\"T15-FREE/object/o9/object_itsbu09_large.png\",\"small_file\":\"T15-FREE/object/o9/object_itsbu09_small.png\"},\"o3\":{\"large_file\":\"T15-FREE/object/o3/object_itsbu03_large.png\",\"small_file\":\"T15-FREE/object/o3/object_itsbu03_small.png\"},\"o1\":{\"large_file\":\"T15-FREE/object/o1/object_jwsbu01_large.png\",\"small_file\":\"T15-FREE/object/o1/object_jwsbu01_small.png\"}},\"scene\":{\"s32\":{\"large_file\":\"T15-FREE/scene/s32/scene_lfsky32_large.png\",\"small_file\":\"T15-FREE/scene/s32/scene_lfsky32_small.png\"},\"s1\":{\"large_file\":\"T15-FREE/scene/s1/scene_lfsample01_large.png\",\"small_file\":\"T15-FREE/scene/s1/scene_lfsample01_small.png\"}}},\"key\":\"T15-FREE\",\"large_file\":\"T15-FREE/T15-Free_large.png\",\"name\":\"T15-FREE\",\"small_file\":\"T15-FREE/T15-Free_small.png\",\"source_path\":\"T15-FREE/T15-FREE.zip\"}}", Project::class.java)
        viewModel.storyLiveData.observe(this, Observer {
            movieEditView.story = it
            moviePlayer.story = it
        })
        viewModel.storyPeriodLiveData.observe(this, Observer {
            movieEditView.period = it ?: 0
            moviePlayer.period = it ?: 0
        })
        viewModel.sceneLiveData.observe(this, Observer {
            movieView.scene = it
        })
        viewModel.musicListLiveData.observe(this, Observer {
            movieEditView.musics = it as ArrayList<Music>
            movieEditView.isAudioTrackGroupViewVisible = it.isNotEmpty()
            moviePlayer.musicList = it
            moviePlayer.init()

        })
        viewModel.musicLiveData.observe(this, Observer {
            viewModel.musicListLiveData.value?.apply {
                movieEditView.currentAudioPosition = indexOf(it)
            }
        })

        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        movieEditView.onPlayClickListener = View.OnClickListener(this::playMovie)
        movieEditView.onPauseClickListener = View.OnClickListener(this::pauseMovie)
        movieEditView.onAddMusicClickListener = View.OnClickListener(this::addMusic)
        movieEditView.onEditMusicClickListener = View.OnClickListener(this::editMusic)
        movieEditView.onDeleteMusicClickListener = View.OnClickListener(this::deleteMusic)
        movieEditView.onSaveMovieClickListener = View.OnClickListener(this::onClickSaveAndShareBtn)
        movieEditView.onSeekBarChangeListener = object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                if (fromUser) {
                    moviePlayer.seekTo(progress.toLong())
                }
            }

            override fun onStartTrackingTouch(seekBar: SeekBar?) {
            }

            override fun onStopTrackingTouch(seekBar: SeekBar?) {
            }
        }
        movieEditView.onMovieSelectListener = object : MovieSelectView.OnMovieSelectListener {
            override fun onMoveSelect(scene: Scene) {
                viewModel.sceneLiveData.value = scene
            }
        }
        movieEditView.onAudioTrackSelectListener = object : AudioTrackGroupView.OnAudioTrackSelectListener {
            override fun onAudioTrackSelect(position: Int) {
                viewModel.musicListLiveData.value?.let {
                    viewModel.musicLiveData.value = it[position]
                }
            }
        }
        movieEditView.onAudioTrackScrollListener = object : AudioTrackGroupView.OnAudioTrackScrollListener {
            override fun onAudioTrackScroll(position: Int, scrollRatio: Double) {
                viewModel.musicListLiveData.value?.get(position)?.offsetTime = ((viewModel.storyPeriodLiveData.value
                        ?: 0) * scrollRatio).toLong()
            }
        }
        movieEditView.onAudioTrackLoopScrollListener = object : AudioTrackGroupView.OnAudioTrackScrollListener {
            override fun onAudioTrackScroll(position: Int, scrollRatio: Double) {
                viewModel.musicListLiveData.value?.get(position)?.loopDuration = ((viewModel.storyPeriodLiveData.value
                        ?: 0) * scrollRatio).toLong()
            }
        }
        movieEditView.onLoopSwitchChangedListener = CompoundButton.OnCheckedChangeListener { _, isChecked ->
            val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
            for (music in musics) {
                music.isLoop = isChecked
            }
            viewModel.musicListLiveData.value = musics
        }

        moviePlayer.moviePlayListener = editMoviePlayListener
    }


    override fun onPostResume() {
        super.onPostResume()
        delayedHide(100)
    }

    private fun delayedHide(delayMillis: Int) {
        mHideHandler.removeCallbacks(mHideRunnable)
        mHideHandler.postDelayed(mHideRunnable, delayMillis.toLong())
    }

    private fun hide() {
        supportActionBar?.hide()
        mHideHandler.removeCallbacks(mShowPart2Runnable)
        mHideHandler.postDelayed(mHidePart2Runnable, UI_ANIMATION_DELAY.toLong())
    }

    private fun playMovie(view: View) {
        moviePlayer.start()
        movieEditView.isBtnPanelEnable = false
    }

    private fun pauseMovie(view: View) {
        moviePlayer.pause()
        movieEditView.isBtnPanelEnable = true
    }

    private fun addMusic(view: View) {
        viewModel.parseDeviceMusic(contentResolver)

        val musicSelectDialog = MusicSelectDialog.newInstance(Gson().toJson(viewModel.musicSource))
        musicSelectDialog.show(supportFragmentManager, MUSIC_SELECT_DIALOG_TAG)
    }

    private fun editMusic(view: View) {
        musicEditDialog = MusicEditDialog.newInstance(Gson().toJson(viewModel.musicLiveData.value))
        musicEditDialog?.show(supportFragmentManager, MUSIC_EDIT_DIALOG_TAG)
    }

    private fun deleteMusic(view: View) {
        val musics: ArrayList<Music> = viewModel.musicListLiveData.value as ArrayList<Music>
        musics.remove(viewModel.musicLiveData.value)
        viewModel.musicListLiveData.value = musics
    }

    private fun onClickSaveAndShareBtn(view: View) {
        shareDialog.apply {
            project = viewModel.project ?: return@apply
            storyPeriod = viewModel.storyPeriodLiveData.value ?: 0
            dismissListener = DialogInterface.OnDismissListener {
                moviePlayer.moviePlayListener = editMoviePlayListener
                moviePlayer.init()

                delayedHide(100)
            }
            seekBarChangedListener = object : SeekBar.OnSeekBarChangeListener {
                override fun onProgressChanged(seekBar: SeekBar?, progress: Int, fromUser: Boolean) {
                    if (fromUser) {
                        moviePlayer.seekTo(progress.toLong())
                    }
                }

                override fun onStartTrackingTouch(seekBar: SeekBar?) {
                }

                override fun onStopTrackingTouch(seekBar: SeekBar?) {
                }
            }
            onClickBomo = Runnable { saveMovie() }
            onClickExportPdf = Runnable { onClickExportPdf() }
            onClickPlay = Runnable {
                moviePlayer.moviePlayListener = dialogMoviePlayListener
                moviePlayer.init()
                moviePlayer.start()
            }
        }.show(supportFragmentManager)
    }

    private fun onClickExportPdf() {
        ExportPdfDialog().also { exportPdfDialog ->
            exportPdfDialog.image1Path = viewModel.project?.scene1File?.path ?: ""
            exportPdfDialog.image2Path = viewModel.project?.scene2File?.path ?: ""
            exportPdfDialog.onClickPrint = Runnable {
                when (exportPdfDialog.type) {
                    ExportPdfDialog.Type.Standard -> {
                        val name = "standard_${viewModel.project?.name}_${System.currentTimeMillis()}.pdf"
                        if (!Config.PDF_FOLDER.exists()) {
                            Config.PDF_FOLDER.mkdirs()
                        }
                        val file = File(Config.PDF_FOLDER, name)
                        file.createNewFile()
                        val observable = PdfMaker.makeStandard(this@MovieEditActivity, viewModel.project!!, file)
                        Pair(file, observable)
                    }
                    ExportPdfDialog.Type.Booklet -> {
                        val name = "booklet_${viewModel.project?.name}_${System.currentTimeMillis()}.pdf"
                        if (!Config.PDF_FOLDER.exists()) {
                            Config.PDF_FOLDER.mkdirs()
                        }
                        val file = File(Config.PDF_FOLDER, name)
                        file.createNewFile()
                        val observable = PdfMaker.makeBooklet(this@MovieEditActivity, viewModel.project!!, file)
                        Pair(file, observable)
                    }
                }.apply {
                    exportPdfFile(second, Runnable {
                        exportPdfDialog.dismiss()
                    }, Runnable {
                        showPdfExportCompleteDialog(first)
                    })
                }
            }
        }.show(supportFragmentManager)
    }

    private fun saveMovie() {
        val scaledWidth = resources.getDimensionPixelSize(R.dimen.movie_width)
        val scaleHeight = resources.getDimensionPixelSize(R.dimen.movie_height)

        val pd = ProgressDialog(this).apply {
            val t = viewModel.storyPeriodLiveData.value ?: 0
            max = (t / 33).toInt()
            setMessage("progressing")
            setCancelable(false)
        }

        val fileName = "test${System.currentTimeMillis()}.mp4"
        val outputFile = File(Environment.getExternalStorageDirectory(), fileName)
        val project = viewModel.project ?: return

        SuperMovieMaker().make(
                this,
                outputFile,
                project,
                viewModel.musicListLiveData.value ?: ArrayList(),
                scaledWidth,
                scaleHeight,
                mediaHelper)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe {
                    pd.show()
                    delayedHide(100)
                }
                .doFinally { pd.dismiss() }
                .subscribe({
                    pd.setMessage(it)
                }, {
                    val msg = """
                    Error:
                    ${it.message}
                """.trimIndent()
                    AlertDialog.Builder(this).setMessage(msg).show()
                    delayedHide(100)
                }, {
                    AlertDialog.Builder(this)
                            .setMessage("Complete")
                            .setPositiveButton("Play") { _, _ ->
                                val uri = Uri.parse(outputFile.absolutePath)
                                val intent = Intent(Intent.ACTION_VIEW, uri).apply {
                                    setDataAndType(uri, "video/mp4")
                                }
                                if (intent.resolveActivity(packageManager) != null) {
                                    Intent.createChooser(intent, "").apply {
                                        startActivity(this)
                                    }
                                } else {
                                    Toast.makeText(this, "No app can play video", Toast.LENGTH_SHORT).show()
                                }
                            }.show()
                    delayedHide(100)
                })
    }

    private fun exportPdfFile(observable: Observable<Float>, onFinally: Runnable, onComplete: Runnable) {
        val pd = ProgressDialog(this@MovieEditActivity).apply {
            setCancelable(false)
        }
        observable.subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .doOnSubscribe { pd.show() }
                .doFinally {
                    pd.dismiss()
                    onFinally.run()
                }
                .subscribe({
                    val progress = (it * 100).toInt()
                    val msg = if (progress >= 100) {
                        "processing ..."
                    } else {
                        "$progress%"
                    }
                    pd.setMessage(msg)
                }, {
                }, {
                    onComplete.run()
                })
    }

    private fun showPdfExportCompleteDialog(file: File) {
        AlertDialog.Builder(this@MovieEditActivity)
                .setMessage("Complete")
                .setPositiveButton("View") { _, _ ->
                    val uri = Uri.parse(file.absolutePath)
                    val intent = Intent(Intent.ACTION_VIEW, uri).apply {
                        setDataAndType(uri, "application/pdf")
                    }
                    if (intent.resolveActivity(packageManager) != null) {
                        Intent.createChooser(intent, "").apply {
                            startActivity(this)
                        }
                    } else {
                        Toast.makeText(this@MovieEditActivity, "No app can view pdf", Toast.LENGTH_SHORT).show()
                    }
                }.show()
    }
}